@@ -1,35 +1,40 @@
+use std::thread;
+
 use crate::{
-    event::{ContextURI, Event, PlayerEvent},
-    state::{Context, PageState, SharedState, Track},
-    token, utils,
+    event::{ClientRequest, ContextURI, PlayerRequest},
+    state::*,
+    token,
+    utils::{self, new_list_state},
 };
 use anyhow::{anyhow, Result};
 use librespot_core::session::Session;
 use rspotify::{blocking::client::Spotify, model::*, senum::*};
 
 /// A spotify client
+#[derive(Clone)]
 pub struct Client {
-    session: Session,
     spotify: Spotify,
     http: reqwest::Client,
 }
 
 impl Client {
     /// creates the new client
-    pub async fn new(session: Session, state: &SharedState) -> Result<Self> {
-        let token = token::get_token(&session, &state.app_config.client_id).await?;
-        let spotify = Spotify::default().access_token(&token.access_token);
-        state.player.write().unwrap().token = token;
-        Ok(Self {
-            session,
-            spotify,
+    pub fn new() -> Self {
+        Self {
+            spotify: Spotify::default(),
             http: reqwest::Client::new(),
-        })
+        }
     }
 
-    /// handles a player event
-    fn handle_player_event(&self, state: &SharedState, event: PlayerEvent) -> Result<()> {
-        log::info!("handle player event: {:?}", event);
+    /// handles a player request
+    fn handle_player_request(&self, state: &SharedState, request: PlayerRequest) -> Result<()> {
+        log::info!("handle player request: {:?}", request);
+
+        // `TransferPlayback` needs to handle separately b/c it doesn't require to
+        // have an active playback
+        if let PlayerRequest::TransferPlayback(device_id, force_play) = request {
+            return self.transfer_playback(device_id, force_play);
+        }
 
         let player = state.player.read().unwrap();
         let playback = match player.playback {
@@ -38,59 +43,59 @@ impl Client {
                 return Err(anyhow!("failed to get the current playback context"));
             }
         };
-        match event {
-            PlayerEvent::NextTrack => self.next_track(playback),
-            PlayerEvent::PreviousTrack => self.previous_track(playback),
-            PlayerEvent::ResumePause => self.resume_pause(playback),
-            PlayerEvent::SeekTrack(position_ms) => self.seek_track(playback, position_ms),
-            PlayerEvent::Repeat => self.repeat(playback),
-            PlayerEvent::Shuffle => self.shuffle(playback),
-            PlayerEvent::Volume(volume) => self.volume(playback, volume),
-            PlayerEvent::PlayTrack(context_uri, track_uris, offset) => {
+
+        match request {
+            PlayerRequest::NextTrack => self.next_track(playback),
+            PlayerRequest::PreviousTrack => self.previous_track(playback),
+            PlayerRequest::ResumePause => self.resume_pause(playback),
+            PlayerRequest::SeekTrack(position_ms) => self.seek_track(playback, position_ms),
+            PlayerRequest::Repeat => self.repeat(playback),
+            PlayerRequest::Shuffle => self.shuffle(playback),
+            PlayerRequest::Volume(volume) => self.volume(playback, volume),
+            PlayerRequest::PlayTrack(context_uri, track_uris, offset) => {
                 self.start_playback(playback, context_uri, track_uris, offset)
             }
+            PlayerRequest::TransferPlayback(..) => unreachable!(),
         }
     }
 
-    /// handles a client event
-    pub async fn handle_event(
-        &mut self,
-        state: &SharedState,
-        send: &std::sync::mpsc::Sender<Event>,
-        event: Event,
-    ) -> Result<()> {
-        log::info!("handle the client event {:?}", event);
+    /// handles a client request
+    pub async fn handle_request(&self, state: &SharedState, request: ClientRequest) -> Result<()> {
+        log::info!("handle the client request {:?}", request);
 
-        match event {
-            Event::Player(event) => {
-                self.handle_player_event(state, event)?;
-                utils::update_playback(state, send);
-            }
-            Event::RefreshToken => {
-                let expires_at = state.player.read().unwrap().token.expires_at;
-                if std::time::Instant::now() > expires_at {
-                    let token =
-                        token::get_token(&self.session, &state.app_config.client_id).await?;
-                    self.spotify = Spotify::default().access_token(&token.access_token);
-                    state.player.write().unwrap().token = token;
+        match request {
+            ClientRequest::Player(event) => {
+                self.handle_player_request(state, event)?;
+
+                // After handling a request that modifies the player's playback,
+                // try to update the playback state by making `n_refreshes` refresh requests.
+                //
+                // # Why needs more than one request to update the playback?
+                // Spotify API may take a while to update with the new changes,
+                // so we may need to make additional requests to ensure that
+                // the current playback is in sync with the latest change.
+                let n_refreshes = state.app_config.n_refreshes_each_playback_update;
+                let delay_duration = std::time::Duration::from_millis(
+                    state.app_config.refresh_delay_in_ms_each_playback_update,
+                );
+
+                for _ in 0..n_refreshes {
+                    std::thread::sleep(delay_duration);
+                    self.update_current_playback_state(state)?;
                 }
             }
-            Event::GetCurrentPlayback => {
+            ClientRequest::GetCurrentPlayback => {
                 self.update_current_playback_state(state)?;
             }
-            Event::TransferPlayback(device_id, force_play) => {
-                self.transfer_playback(device_id, force_play)?;
-                utils::update_playback(state, send);
-            }
-            Event::GetDevices => {
+            ClientRequest::GetDevices => {
                 let devices = self.get_devices()?;
                 state.player.write().unwrap().devices = devices;
             }
-            Event::GetUserPlaylists => {
+            ClientRequest::GetUserPlaylists => {
                 let playlists = self.get_current_user_playlists().await?;
                 state.player.write().unwrap().user_playlists = playlists;
             }
-            Event::GetUserFollowedArtists => {
+            ClientRequest::GetUserFollowedArtists => {
                 let artists = self
                     .get_current_user_followed_artists()
                     .await?
@@ -99,7 +104,7 @@ impl Client {
                     .collect::<Vec<_>>();
                 state.player.write().unwrap().user_followed_artists = artists;
             }
-            Event::GetUserSavedAlbums => {
+            ClientRequest::GetUserSavedAlbums => {
                 let albums = self
                     .get_current_user_saved_albums()
                     .await?
@@ -108,7 +113,7 @@ impl Client {
                     .collect::<Vec<_>>();
                 state.player.write().unwrap().user_saved_albums = albums;
             }
-            Event::GetContext(context) => {
+            ClientRequest::GetContext(context) => {
                 match context {
                     ContextURI::Playlist(playlist_uri) => {
                         self.get_playlist_context(playlist_uri, state).await?;
@@ -129,6 +134,9 @@ impl Client {
                     }
                 };
             }
+            ClientRequest::Search(query) => {
+                self.search(state, query)?;
+            }
         };
 
         Ok(())
@@ -338,6 +346,153 @@ impl Client {
         Self::handle_rspotify_result(self.spotify.current_playback(None, None))
     }
 
+    /// searchs for items (tracks, artists, albums, playlists) that match a given query string.
+    pub fn search(&self, state: &SharedState, query: String) -> Result<()> {
+        // searching for tracks, artists, albums, and playlists that match
+        // a given query string. Each class of items will be handled separately
+        // in a separate thread.
+        let update_ui_states = |results: SearchResults| {
+            let mut ui = state.ui.lock().unwrap();
+            if let PageState::Searching(_, ref mut state_results) = ui.current_page_mut() {
+                *state_results = Box::new(results);
+                ui.window = WindowState::Search(
+                    new_list_state(),
+                    new_list_state(),
+                    new_list_state(),
+                    new_list_state(),
+                    SearchFocusState::Input,
+                );
+            }
+        };
+
+        // already search the query before, updating the ui page state directly
+        if let Some(search_results) = state.player.read().unwrap().search_cache.peek(&query) {
+            update_ui_states(search_results.clone());
+            return Ok(());
+        }
+
+        let tracks_thread = thread::spawn({
+            let spotify = self.spotify.clone();
+            let query = query.clone();
+            move || -> Result<_> {
+                let search_result = Self::handle_rspotify_result(spotify.search(
+                    &query,
+                    SearchType::Track,
+                    None,
+                    None,
+                    None,
+                    None,
+                ))?;
+
+                Ok(match search_result {
+                    search::SearchResult::Tracks(page) => page,
+                    _ => unreachable!(),
+                })
+            }
+        });
+
+        let artists_thread = thread::spawn({
+            let spotify = self.spotify.clone();
+            let query = query.clone();
+            move || -> Result<_> {
+                let search_result = Self::handle_rspotify_result(spotify.search(
+                    &query,
+                    SearchType::Artist,
+                    None,
+                    None,
+                    None,
+                    None,
+                ))?;
+
+                Ok(match search_result {
+                    search::SearchResult::Artists(page) => page,
+                    _ => unreachable!(),
+                })
+            }
+        });
+
+        let albums_thread = thread::spawn({
+            let spotify = self.spotify.clone();
+            let query = query.clone();
+            move || -> Result<_> {
+                let search_result = Self::handle_rspotify_result(spotify.search(
+                    &query,
+                    SearchType::Album,
+                    None,
+                    None,
+                    None,
+                    None,
+                ))?;
+
+                Ok(match search_result {
+                    search::SearchResult::Albums(page) => page,
+                    _ => unreachable!(),
+                })
+            }
+        });
+
+        let playlists_thread = thread::spawn({
+            let spotify = self.spotify.clone();
+            let query = query.clone();
+            move || -> Result<_> {
+                let search_result = Self::handle_rspotify_result(spotify.search(
+                    &query,
+                    SearchType::Playlist,
+                    None,
+                    None,
+                    None,
+                    None,
+                ))?;
+
+                Ok(match search_result {
+                    search::SearchResult::Playlists(page) => page,
+                    _ => unreachable!(),
+                })
+            }
+        });
+
+        let tracks = tracks_thread.join().unwrap()?;
+        let artists = artists_thread.join().unwrap()?;
+        let albums = albums_thread.join().unwrap()?;
+        let playlists = playlists_thread.join().unwrap()?;
+
+        let search_results = SearchResults {
+            tracks: Self::into_page(tracks),
+            artists: Self::into_page(artists),
+            albums: Self::into_page(albums),
+            playlists,
+        };
+
+        // update the search cache stored inside the player state
+        state
+            .player
+            .write()
+            .unwrap()
+            .search_cache
+            .put(query, search_results.clone());
+
+        update_ui_states(search_results);
+        Ok(())
+    }
+
+    /// converts a page of items with type `Y` into a page of items with type `X`
+    /// given that type `Y` can be converted to type `X` through the `Into<X>` trait
+    fn into_page<X, Y: Into<X>>(page_y: page::Page<Y>) -> page::Page<X> {
+        page::Page {
+            items: page_y
+                .items
+                .into_iter()
+                .map(|y| y.into())
+                .collect::<Vec<_>>(),
+            href: page_y.href,
+            limit: page_y.limit,
+            next: page_y.next,
+            offset: page_y.offset,
+            previous: page_y.previous,
+            total: page_y.total,
+        }
+    }
+
     /// handles a `rspotify` client result and converts it into `anyhow` compatible result format
     fn handle_rspotify_result<T, E: std::fmt::Display>(
         result: std::result::Result<T, E>,
@@ -562,32 +717,73 @@ impl Client {
     }
 }
 
-/// starts the client's event handler
+/// starts the client's request handler
 #[tokio::main]
 pub async fn start_client_handler(
     state: SharedState,
     mut client: Client,
-    send: std::sync::mpsc::Sender<Event>,
-    recv: std::sync::mpsc::Receiver<Event>,
+    recv: std::sync::mpsc::Receiver<ClientRequest>,
 ) {
-    while let Ok(event) = recv.recv() {
-        if let Err(err) = client.handle_event(&state, &send, event).await {
-            log::warn!("{:#?}", err);
+    while let Ok(request) = recv.recv() {
+        // use the authentication token stored inside the player state
+        // for making API calls to Spotify
+        client.spotify.access_token =
+            Some(state.player.read().unwrap().token.access_token.to_owned());
+
+        // This is not a nice way to handle a request without blocking :/.
+        // Kinda a workaround atm.
+        // TODO: find a better approach to handle a request without blocking and,
+        // at the same time, minimizing the number of clones
+        {
+            // handle the client request while trying not to block the current thread
+            let client = client.clone();
+            let state = state.clone();
+            tokio::spawn(async move {
+                if let Err(err) = client.handle_request(&state, request).await {
+                    log::warn!("{:#?}", err);
+                }
+            });
         }
     }
 }
 
-// starts multiple threads watching for player events then
-// notifying the client to make update requests
-pub fn start_player_event_watchers(
+// starts multiple event watchers that
+// listen to player events and notify the client
+// to make additional update requests if needed
+#[tokio::main]
+pub async fn start_player_event_watchers(
     state: SharedState,
-    send: std::sync::mpsc::Sender<Event>,
-) -> Result<()> {
-    // request devices and playback data for rendering the UI on startup
-    send.send(Event::GetCurrentPlayback)?;
-    send.send(Event::GetDevices)?;
+    send: std::sync::mpsc::Sender<ClientRequest>,
+    session: Session,
+) {
+    std::thread::spawn({
+        let send = send.clone();
+        move || {
+            // need to get the playback data on startup to render the player
+            //
+            // - Why needs to query the APIs multiple times?
+            // On startup, the integrated librespot device (enabled by the "streaming" feature)
+            // may not be initialized at the time the outer function is called.
+            // Querying the APIs multiple times is kinda a workaround to ensure
+            // that the initial playback data is in sync.
+            for _ in 0..5 {
+                // on startup, the UI needs to know the current playback to render the current playing context
+                send.send(ClientRequest::GetCurrentPlayback)
+                    .unwrap_or_else(|err| {
+                        log::warn!("failed to get the current playback: {}", err);
+                    });
+                // needs to know all available devices on startup to connect to the first available device if none is running
+                send.send(ClientRequest::GetDevices).unwrap_or_else(|err| {
+                    log::warn!("failed to get devices: {}", err);
+                });
+
+                std::thread::sleep(std::time::Duration::from_millis(1000));
+            }
+        }
+    });
 
     // start a playback pooling (every `playback_refresh_duration_in_ms` ms) thread
+    // A zero-value `playback_refresh_in_ms` indicates no playback pooling thread
     if state.app_config.playback_refresh_duration_in_ms > 0 {
         std::thread::spawn({
             let send = send.clone();
@@ -595,114 +791,133 @@ pub fn start_player_event_watchers(
                 std::time::Duration::from_millis(state.app_config.playback_refresh_duration_in_ms);
             move || -> Result<()> {
                 loop {
-                    send.send(Event::GetCurrentPlayback)?;
+                    send.send(ClientRequest::GetCurrentPlayback)
+                        .unwrap_or_else(|err| {
+                            log::warn!("failed to get the current playback: {}", err);
+                        });
                     std::thread::sleep(playback_refresh_duration);
                 }
             }
         });
     }
 
-    // start the main player event watcher thread
-    std::thread::spawn(move || -> Result<()> {
-        let refresh_duration = std::time::Duration::from_millis(1000);
-        loop {
-            {
-                // get the current UI's page state for updating player's context on change
-                let page = state.ui.lock().unwrap().page.clone();
-                let player = state.player.read().unwrap();
+    // start the main event watcher watching for new events every second
+    let refresh_duration = std::time::Duration::from_millis(1000);
+    loop {
+        watch_player_events(&state, &send, &session)
+            .await
+            .unwrap_or_else(|err| {
+                log::warn!(
+                    "encountered an error when watching for player events: {}",
+                    err
+                );
+            });
 
-                // update the auth token if expired
-                if std::time::Instant::now() > player.token.expires_at {
-                    send.send(Event::RefreshToken)?;
-                }
+        std::thread::sleep(refresh_duration);
+    }
+}
 
-                {
-                    // if cannot find the current playback, try
-                    // to connect the first avaiable device
-                    if player.playback.is_none() && !player.devices.is_empty() {
-                        log::info!(
-                            "no playback found, try to connect device {}",
-                            player.devices[0].name
-                        );
-                        send.send(Event::TransferPlayback(player.devices[0].id.clone(), false))?;
-                    }
-                }
+async fn watch_player_events(
+    state: &SharedState,
+    send: &std::sync::mpsc::Sender<ClientRequest>,
+    session: &Session,
+) -> Result<()> {
+    // update the auth token if expired
+    if std::time::Instant::now() > state.player.read().unwrap().token.expires_at {
+        log::info!("auth token inside the player state is expired, trying to get a new token...");
+        let token = token::get_token(session, &state.app_config.client_id).await?;
+        state.player.write().unwrap().token = token;
+    }
 
-                {
-                    // refresh the playback when the current song ends
-                    let progress_ms = player.get_playback_progress();
-                    let duration_ms = player.get_current_playing_track().map(|t| t.duration_ms);
-                    let is_playing = match player.playback {
-                        Some(ref playback) => playback.is_playing,
-                        None => false,
-                    };
-                    if let (Some(progress_ms), Some(duration_ms)) = (progress_ms, duration_ms) {
-                        if progress_ms >= duration_ms && is_playing {
-                            send.send(Event::GetCurrentPlayback)?;
-                        }
-                    }
-                }
+    {
+        let player = state.player.read().unwrap();
 
-                // update the player's context based on the UI's page state
-                match page {
-                    PageState::Browsing(uri) => {
-                        if player.context_uri != uri {
-                            utils::update_context(&state, uri);
-                        }
-                    }
-                    PageState::CurrentPlaying => {
-                        // updates the context (album, playlist, etc) tracks based on the current playback
-                        if let Some(ref playback) = player.playback {
-                            match playback.context {
-                                Some(ref context) => {
-                                    let uri = context.uri.clone();
+        // if cannot find the current playback, try
+        // to connect the first avaiable device
+        if player.playback.is_none() && !player.devices.is_empty() {
+            log::info!(
+                "no playback found, try to connect the first available device {}",
+                player.devices[0].name
+            );
+            // only trying to connect, not transfer the current playback
+            send.send(ClientRequest::Player(PlayerRequest::TransferPlayback(
+                player.devices[0].id.clone(),
+                false,
+            )))?;
+        }
 
-                                    if uri != player.context_uri {
-                                        utils::update_context(&state, uri.clone());
-                                        if player.context_cache.peek(&uri).is_none() {
-                                            match context._type {
-                                                rspotify::senum::Type::Playlist => send.send(
-                                                    Event::GetContext(ContextURI::Playlist(uri)),
-                                                )?,
-                                                rspotify::senum::Type::Album => send.send(
-                                                    Event::GetContext(ContextURI::Album(uri)),
-                                                )?,
-                                                rspotify::senum::Type::Artist => send.send(
-                                                    Event::GetContext(ContextURI::Artist(uri)),
-                                                )?,
-                                                _ => {
-                                                    send.send(Event::GetContext(
-                                                        ContextURI::Unknown(uri),
-                                                    ))?;
-                                                    log::info!(
-                                                    "encountered not supported context type: {:#?}",
-                                                    context._type
-                                                )
-                                                }
-                                            };
-                                        }
-                                    }
-                                }
-                                None => {
-                                    if !player.context_uri.is_empty() {
-                                        utils::update_context(&state, "".to_string());
-                                        send.send(Event::GetContext(ContextURI::Unknown(
-                                            "".to_string(),
+        // update the playback when the current track ends
+        let progress_ms = player.get_playback_progress();
+        let duration_ms = player.get_current_playing_track().map(|t| t.duration_ms);
+        let is_playing = match player.playback {
+            Some(ref playback) => playback.is_playing,
+            None => false,
+        };
+        if let (Some(progress_ms), Some(duration_ms)) = (progress_ms, duration_ms) {
+            if progress_ms >= duration_ms && is_playing {
+                send.send(ClientRequest::GetCurrentPlayback)?;
+            }
+        }
+    }
+
+    // update the player's context based on the UI's page state
+    match state.ui.lock().unwrap().current_page() {
+        PageState::Searching(..) => {
+            // manually empty the `context_uri` to trigger
+            // context updating when moving from the search page
+            // to a previous context page and vice versa
+            state.player.write().unwrap().context_uri = "".to_owned();
+        }
+        PageState::Browsing(uri) => {
+            if state.player.read().unwrap().context_uri != *uri {
+                utils::update_context(state, uri.clone());
+            }
+        }
+        PageState::CurrentPlaying => {
+            let player = state.player.read().unwrap();
+            // updates the context (album, playlist, etc) tracks based on the current playback
+            if let Some(ref playback) = player.playback {
+                match playback.context {
+                    Some(ref context) => {
+                        let uri = context.uri.clone();
+
+                        if uri != player.context_uri {
+                            utils::update_context(state, uri.clone());
+                            if player.context_cache.peek(&uri).is_none() {
+                                match context._type {
+                                    rspotify::senum::Type::Playlist => send.send(
+                                        ClientRequest::GetContext(ContextURI::Playlist(uri)),
+                                    )?,
+                                    rspotify::senum::Type::Album => send
+                                        .send(ClientRequest::GetContext(ContextURI::Album(uri)))?,
+                                    rspotify::senum::Type::Artist => send
+                                        .send(ClientRequest::GetContext(ContextURI::Artist(uri)))?,
+                                    _ => {
+                                        send.send(ClientRequest::GetContext(ContextURI::Unknown(
+                                            uri,
                                         )))?;
                                         log::info!(
-                                            "current playback does not have a playing context"
-                                        );
+                                            "encountered not supported context type: {:#?}",
+                                            context._type
+                                        )
                                     }
-                                }
+                                };
                             }
-                        };
+                        }
+                    }
+                    None => {
+                        if !player.context_uri.is_empty() {
+                            utils::update_context(state, "".to_string());
+                            send.send(ClientRequest::GetContext(ContextURI::Unknown(
+                                "".to_string(),
+                            )))?;
+                            log::info!("current playback does not have a playing context");
+                        }
                     }
                 }
-            }
-
-            std::thread::sleep(refresh_duration);
+            };
         }
-    });
+    }
 
     Ok(())
 }
