@@ -16,8 +16,8 @@ pub enum Command {
     OpenCommandHelp,
     ClosePopup,
 
-    SelectNext,
-    SelectPrevious,
+    SelectNextOrScrollDown,
+    SelectPreviousOrScrollUp,
     ChooseSelected,
 
     RefreshPlayback,
@@ -39,6 +39,8 @@ pub enum Command {
     BrowseSelectedTrackArtists,
     BrowseSelectedTrackAlbum,
 
+    SearchPage,
+
     PreviousPage,
 
     SortTrackByTitle,
@@ -63,8 +65,10 @@ impl Command {
             Self::Quit => "quit the application",
             Self::OpenCommandHelp => "open a command help popup",
             Self::ClosePopup => "close a popup",
-            Self::SelectNext => "select the next item in the focused list or table",
-            Self::SelectPrevious => "select the previous item in the focused list or table",
+            Self::SelectNextOrScrollDown => "select the next item in a list/table or scroll down",
+            Self::SelectPreviousOrScrollUp => {
+                "select the previous item in a list/table or scroll up"
+            }
             Self::ChooseSelected => "choose the selected item and act on it",
             Self::RefreshPlayback => "manually refresh the current playback",
             Self::FocusNextWindow => "focus the next focusable window (if any)",
@@ -84,6 +88,7 @@ impl Command {
                 "open a popup for browsing the selected track's artists"
             }
             Self::BrowseSelectedTrackAlbum => "browse to the selected track's album",
+            Self::SearchPage => "go to the search page",
             Self::PreviousPage => "go to the previous page",
             Self::SortTrackByTitle => "sort the track table (if any) by track's title",
             Self::SortTrackByArtists => "sort the track table (if any) by track's artists",
