@@ -14,6 +14,8 @@ pub struct PlayerState {
     pub context_uri: String,
     pub context_cache: lru::LruCache<String, Context>,
 
+    pub search_cache: lru::LruCache<String, SearchResults>,
+
     pub playback: Option<context::CurrentlyPlaybackContext>,
     pub playback_last_updated: Option<std::time::Instant>,
 }
@@ -27,6 +29,20 @@ pub enum Context {
     Unknown(String),
 }
 
+/// SearchResults denotes the returned data when searching using Spotify API.
+/// Search results are returned as pages of Spotify objects, which includes
+/// - `tracks`
+/// - `albums`
+/// - `artists`
+/// - `playlists`
+#[derive(Clone, Debug)]
+pub struct SearchResults {
+    pub tracks: page::Page<Track>,
+    pub artists: page::Page<Artist>,
+    pub albums: page::Page<Album>,
+    pub playlists: page::Page<playlist::SimplifiedPlaylist>,
+}
+
 #[derive(Debug)]
 /// Order of sorting tracks in a playing context
 pub enum ContextSortOrder {
@@ -106,6 +122,30 @@ impl PlayerState {
     }
 }
 
+impl SearchResults {
+    fn empty_page<T>() -> page::Page<T> {
+        page::Page {
+            href: "".to_owned(),
+            items: vec![],
+            limit: 0,
+            next: None,
+            offset: 0,
+            previous: None,
+            total: 0,
+        }
+    }
+
+    // returns an empty search results
+    pub fn empty() -> Self {
+        Self {
+            tracks: Self::empty_page::<Track>(),
+            artists: Self::empty_page::<Artist>(),
+            albums: Self::empty_page::<Album>(),
+            playlists: Self::empty_page::<playlist::SimplifiedPlaylist>(),
+        }
+    }
+}
+
 impl Default for PlayerState {
     fn default() -> Self {
         Self {
@@ -116,6 +156,7 @@ impl Default for PlayerState {
             user_followed_artists: vec![],
             context_uri: "".to_owned(),
             context_cache: lru::LruCache::new(64),
+            search_cache: lru::LruCache::new(64),
             playback: None,
             playback_last_updated: None,
         }
@@ -145,11 +186,25 @@ impl Context {
             Context::Unknown(_) => {
                 "Cannot infer the playing context from the current playback".to_owned()
             }
-            Context::Album(ref album, _) => {
-                format!("Album: {}", album.name)
+            Context::Album(ref album, ref tracks) => {
+                format!(
+                    "Album: {} | {} | {} songs",
+                    album.name,
+                    album.release_date,
+                    tracks.len()
+                )
             }
-            Context::Playlist(ref playlist, _) => {
-                format!("Playlist: {}", playlist.name)
+            Context::Playlist(ref playlist, tracks) => {
+                format!(
+                    "Playlist: {} | {} | {} songs",
+                    playlist.name,
+                    playlist
+                        .owner
+                        .display_name
+                        .as_ref()
+                        .unwrap_or(&"unknown".to_owned()),
+                    tracks.len()
+                )
             }
             Context::Artist(ref artist, _, _, _) => {
                 format!("Artist: {}", artist.name)
